#ifndef TKDESTEP_PCH_H
#define TKDESTEP_PCH_H

#include <StepData_StepReaderData.hxx>
#include <StepData_StepWriter.hxx>
#include <StepGeom_GeometricRepresentationItem.hxx>
#include <StepKinematics_KinematicPair.hxx>
#include <StepRepr_Representation.hxx>
#include <StepRepr_RepresentationItem.hxx>

#endif // TKDESTEP_PCH_H
